@@ -1,4 +1,4 @@
-globalVariables(c("bound_end", "bound_start", "est", "label", "lwr", "stim_end", "stim_start", "upr", "vbl", "i", "j", "vij", "zb"))
+globalVariables(c("bound_end", "bound_start", "est", "label", "lwr", "stim_end", "stim_start", "upr", "vbl", "i", "j", "vij", "zb", "smaller", "larger", "ll", "us", "olap", "ambig", "lwr_add", "upr_add"))
 
 #' Calculate Correspondence Between Pairwise Test and CI Overlaps
 #' 
@@ -32,12 +32,15 @@ globalVariables(c("bound_end", "bound_start", "est", "label", "lwr", "stim_end",
 #' @param include_intercept Logical indicating whether the intercept should be included in the tests, defaults to `FALSE`.
 #' @param include_zero Should univariate tests at zero be included, defaults to `TRUE`.
 #' @param sig_diffs An optional vector of values identify whether each pair of values is statistically different (1) or not (0).  See Details for more information on specifying this value; there is some added complexity here. 
+#' @param tol Tolerance for evaluation of symmetry and positive definiteness. 
 #' @param ... Other arguments, currently not implemented.
 #' @export
 #' 
 #' @references David A. Armstrong II and William Poirier. "Decoupling Visualization and Testing when Presenting Confidence Intervals" Political Analysis <doi:10.1017/pan.2024.24>.
 #' @importFrom stats coef vcov qt pt p.adjust
 #' @importFrom utils combn
+#' @importFrom multcomp glht adjusted
+#' @importFrom dplyr left_join 
 #' @returns A list (of class "viztest") with the following elements: 
 #' 1. tab: a data frame with results from the grid search.  The data frame has four variables: `level` - is the confidence level used in the grid search; `psame` - the proportion of (non-)overlaps that match the 
 #' normal theory tests; `pdiff` - the proportion of pairwise tests that are statistically significant; `easy` - the ease with which the comparisons are made. 
@@ -48,6 +51,7 @@ globalVariables(c("bound_end", "bound_start", "est", "label", "lwr", "stim_end",
 #' 6. L: The lower confidence bounds from the grid search. 
 #' 7. U: The upper confidence bounds from the grid search. 
 #' 8. est: A data frame with the variables `vbl` - the parameter name; `est` - the parameter estimate; `se` - the parameter standard error. 
+#' 9. call: model call
 #' @examples
 #' data(mtcars)
 #' mtcars$cyl <- as.factor(mtcars$cyl)
@@ -65,6 +69,7 @@ viztest <- function(obj,
                     include_intercept = FALSE,
                     include_zero = TRUE,
                     sig_diffs = NULL,
+                    tol = 1e-08,
                     ...){
   UseMethod("viztest")
 }
@@ -82,7 +87,9 @@ viztest.default <- function(obj,
                      include_intercept = FALSE,
                      include_zero = TRUE,
                      sig_diffs = NULL,
+                     tol = 1e-08,
                      ...){
+  
   adj <- match.arg(adjust)
   lev_seq <- seq(range_levels[1], range_levels[2], by=level_increment)
   resdf <- Inf
@@ -92,6 +99,28 @@ viztest.default <- function(obj,
   bhat <- coef(obj)
   if(is.null(names(bhat)))names(bhat) <- 1:length(bhat)
   V <- vcov(obj)
+  if(any(is.na(bhat))){
+    na_coef <- which(is.na(bhat))
+    if(inherits(V, "matrix")){
+      na_var <- which(apply(V, 1, \(x)all(is.na(x))))
+    }else{
+      na_var <- which(is.na(V))
+    }
+    coef_var_agree <- try(all(na_var == na_coef), silent=TRUE)
+    if(!inherits(coef_var_agree, "try-error")){
+      bhat <- bhat[-na_coef]
+      V <- V[-na_var, -na_var]
+      message(paste0("Rank-deficient model detected - parameter name(s):", paste(names(na_coef), collapse=", "), " removing NAs from estimates/variances and proceeding.\n"))
+    }else{
+      stop("NAs found in estimates and/or variances, but they are not in compatible places, please solve the problem and try again.\n")
+    }
+  }
+  if(!is_pd(V, tol=tol)){
+    stop("Variance-covariance matrix is not positive definite.  Cannot proceed with viztest().\n")
+  }
+  if(!(length(bhat) == nrow(V) & is_sym(V, tol=tol))){
+    stop("Length of coefficient vector does not match dimensions of variance-covariance matrix.  Cannot proceed with viztest().\n")
+  }
   if(!include_intercept){
     w_int <- grep("ntercept", names(bhat))
     if(length(w_int) > 0){
@@ -114,7 +143,7 @@ viztest.default <- function(obj,
     D[cbind(combs[2,], 1:ncol(combs))] <- -1
     diffs <- bhat %*% D
     se_diffs <- sqrt(diag(t(D) %*% V %*% D))
-    p_diff <- pt(diffs/se_diffs, resdf, lower.tail=FALSE)
+    p_diff <- 2*pt(diffs/se_diffs, resdf, lower.tail=FALSE)
     p_diff <- p.adjust(p_diff, method=adj)
     s <- p_diff < test_level
   }else{
@@ -155,6 +184,29 @@ viztest.default <- function(obj,
   est_data <- tibble(vbl = names(bhat), 
                      est = bhat, 
                      se = sqrt(diag(V)))
+  if(adj == "none"){
+    est_data$lwr_add <- bhat - qt(1-test_level/2, resdf)*sqrt(diag(V))
+    est_data$upr_add <- bhat + qt(1-test_level/2, resdf)*sqrt(diag(V))
+  }else{
+    if("zero" %in% names(bhat)){
+      tmp_bhat <- bhat
+      tmp_V <- V
+      zero_ind <- which(names(bhat) == "zero")
+      tmp_bhat <- bhat[-zero_ind]
+      tmp_V <- V[-zero_ind, -zero_ind]
+    }else {
+      tmp_bhat <- bhat
+      tmp_V <- V
+    }
+    K <- diag(length(tmp_bhat))
+    g <- glht(model = NULL, linfct = K, coef.=tmp_bhat, vcov.=tmp_V)
+    smry <- summary(g, test=adjusted(adj))
+    cis <- confint(smry)
+    dat_add <- data.frame(vbl = names(tmp_bhat), 
+                          lwr_add = cis$confint[,"lwr"], 
+                          upr_add = cis$confint[,"upr"])
+    est_data <- left_join(est_data, dat_add, by="vbl")
+  }    
   res <- list(tab = res,
               pw_test = s,
               ci_tests = s_star,
@@ -162,8 +214,11 @@ viztest.default <- function(obj,
               param_names = names(bhat),
               L = L,
               U = U, 
-              est = est_data)
+              est = est_data, 
+              call = match.call(viztest, call = sys.call()))
   class(res) <- "viztest"
+  attr(res, "adjusted") <- adj
+  attr(res, "test_level") <- test_level 
   return(res)
 }
 #' @importFrom stats quantile
@@ -178,7 +233,8 @@ viztest.vtsim <- function(obj,
                           cifun = c("quantile", "hdi"), 
                           include_intercept = FALSE,
                           include_zero = TRUE,
-                          sig_diffs = NULL, 
+                          sig_diffs = NULL,
+                          tol = 1e-08, 
                           ...){
   cif <- match.arg(cifun)
   est <- obj$est
@@ -195,7 +251,7 @@ viztest.vtsim <- function(obj,
     D[cbind(combs[2,], 1:ncol(combs))] <- -1
     diffs <- est %*% D
     pvals <- apply(diffs, 2, \(x)mean(x > 0))
-    pvals <- ifelse(pvals > .5, 1-pvals, pvals)
+    pvals <- 2*ifelse(pvals > .5, 1-pvals, pvals)
     s <- pvals < test_level
   }else{
     s <- sig_diffs
@@ -204,10 +260,15 @@ viztest.vtsim <- function(obj,
   if(cif == "quantile"){
     L <- sapply(lev_seq, \(l)apply(est, 2, quantile, probs=((1-l)/2)))
     U <- sapply(lev_seq, \(l)apply(est, 2, quantile, probs=(1-(1-l)/2)))
+    tl_L <- apply(est, 2, quantile, probs=(1 - (1 - test_level/2)))
+    tl_U <- apply(est, 2, quantile, probs=(1 - test_level / 2))
   }else{
     LU <- lapply(lev_seq, \(l)apply(est, 2, hdi, credMass = l))
     L <- sapply(LU, \(x)x[1,])
     U <- sapply(LU, \(x)x[2,])
+    tl_LU <- apply(est, 2,  hdi, credMass = 1-test_level)
+    tl_L <- tl_LU[1,]
+    tl_U <- tl_LU[2,]
   }
   s_star <- L[combs[1,], , drop=FALSE] >= U[combs[2,], , drop=FALSE]
   smat <- array(s, dim=dim(s_star))
@@ -243,6 +304,8 @@ viztest.vtsim <- function(obj,
   est_data <- tibble(vbl = names(cme), 
                      est = cme, 
                      sd = esd)
+  est_data$lwr_add <- tl_L
+  est_data$upr_add <- tl_U
   res <- list(tab = res,
               pw_test = s,
               ci_tests = s_star,
@@ -250,8 +313,11 @@ viztest.vtsim <- function(obj,
               param_names = colnames(est),
               L = L,
               U = U, 
-              est = est_data)
+              est = est_data, 
+              call = match.call(viztest, call = sys.call()))
   class(res) <- "viztest"
+  attr(res, "adjusted") <- "none"
+  attr(res, "test_level") <- test_level 
   return(res)
 }
 
@@ -390,11 +456,12 @@ print.viztest <- function(x, ..., best=TRUE, missed_tests=TRUE, level=NULL){
 #' raw variances from the data. 
 #' @param type Indicates the type of input data either estimates with variances or a variance-covariance matrix or data from
 #' a simulation. 
+#' @param tol Tolerance for evaluation of symmetry and positive definiteness. 
 #' @param ... Other arguments passed down, currently not implemented. 
-#' @returns An object of class "vtcustom" that takes one of two forms: 
-#' 1. A list with estimates and a variance-covariance matrix.  In this case, the functionms `coef.vtcustom()` and `vcov.vtcustom()` are 
+#' @returns 
+#' 1. If the input is a vector of parameter estimates and a variance-covariance matrix, then a list with estimates and a variance-covariance matrix of class `"vtcustom"` is returned.  In this case, the functionms `coef.vtcustom()` and `vcov.vtcustom()` are 
 #' used to extract the coefficients and variance-covariance matrix in a way that will work with `viztest.default()`. 
-#' 2. An object of class "vtsim" that has a single element - the data giving the draws from the simulation. 
+#' 2. If the input is a matrix of simulation draws, an object of class `"vtsim"` that has a single element - the data giving the draws from the simulation is returned.  In this case, `viztest.vtsim()` does the relevant testing.  
 #' @export
 #' 
 #' @examples
@@ -409,7 +476,7 @@ print.viztest <- function(x, ..., best=TRUE, missed_tests=TRUE, level=NULL){
 #'         test_level = .025, 
 #'         include_intercept = FALSE, 
 #'         include_zero = FALSE)
-make_vt_data <- function(estimates, variances=NULL, type=c("est_var", "sim"), ...){
+make_vt_data <- function(estimates, variances=NULL, type=c("est_var", "sim"), tol = 1e-08, ...){
   typ <- match.arg(type)
   if(typ == "est_var"){
     if(is.null(variances)){
@@ -423,11 +490,30 @@ make_vt_data <- function(estimates, variances=NULL, type=c("est_var", "sim"), ..
         stop("Variance-covariance matrix must have the same number of rows and columns as there are elements in the estimates vector.\n")
       }
     }
+    if(any(is.na(estimates))){
+      na_coef <- which(is.na(estimates))
+      if(inherits(variances, "matrix")){
+        na_var <- which(apply(variances, 1, \(x)all(is.na(x))))
+      }else{
+        na_var <- which(is.na(variances))
+      }
+      coef_var_agree <- try(all(na_var == na_coef), silent=TRUE)
+      if(!inherits(coef_var_agree, "try-error")){
+        estimates <- estimates[-na_coef]
+        variances <- variances[-na_var, -na_var]
+        message(paste0("Rank-deficient model detected - parameter name(s):", paste(names(na_coef), collapse=", "), " removing NAs from estimates/variances and proceeding.\n"))
+      }else{
+        stop("NAs found in estimates and/or variances, but they are not in compatible places, please solve the problem and try again.\n")
+      }
+    }
     if(!inherits(variances, "matrix")){
       V <- diag(variances)
     }else{
       V <- variances
     }
+    if(!is_pd(V, tol=tol)){
+      stop("Variance-covariance matrix is not positive definite.  Cannot proceed with viztest().\n")
+    }
     out <- structure(.Data = list(coef = estimates, vcov = V), class="vtcustom")
   }else{
     if(!is.null(variances)){
@@ -479,11 +565,21 @@ make_segs <- function(.data, vdt = .02, ...){
 #' 
 #' Plots the output of viztest objects with optional reference lines 
 #' @param x Object to be plotted, should be of class `viztest`
+#' @param add_test_level Add the (1-test level) confidence interval to the plot.  For this to work, you must have specified `add_test_level` in the call to `viztest()` so that the appropriate confidence intervals can be calculated.  
 #' @param ref_lines Reference lines to be plotted - one of "all", "ambiguous", "none".  This could also be a vector of stimulus names to plot - they should be the same as the names of the estimates in `x$est`. See details for explanation. 
 #' @param viz_diff_thresh Threshold for identifying visual difficulty, see details. 
 #' @param make_plot Logical indicating whether the plot should be constructed or the data returned. 
 #' @param level Level at which to plot the estimates.  Accepts both numeric entries or one of "ce", "max", "min", "median" - defaults to "ce", the cognitively easiest level.  
 #' @param trans A function to transform the estimates and their confidence intervals like `plogis`.
+#' @param est_point_args A list of arguments to be passed to `geom_point()` that plots the point estimates. 
+#' @param opt_ci_args A list of arguments to be passed to `geom_linerange()` to plot the optimal visual testing intervals. 
+#' @param test_ci_args A list of arguments to be passed to `geom_linerange()` to plot the (1-test level) confidence intervals.
+#' @param ref_line_args A list of arguments to be passed to `geom_segment()` to plot the reference lines.
+#' @param scale_linewidth_args A list of arguments to be passed to `scale_linewidth_manual()` to change the thickness of the confidence intervals.
+#' @param scale_color_args A list of arguments to be passed to `scale_color_manual()` to change the default color of the different confidence intervals when `add_test_level=TRUE`.
+#' @param overall_theme A theme function that will be passed to the `ggplot` call before `theme()`. Default is `theme_bw`.
+#' @param theme_arg A list of arguments to be passed to `theme()` to modify the theme of the plot.
+#' @param remove_caption Logical indicating whether caption should be removed.  By default, it is printed to alert the user. 
 #' @param ... Other arguments passed down.  Currently not implemented.
 #' @details The `ref_lines` argument identifies what reference lines will be plotted in the figure.  For any particular stimulus, the reference lines run along the upper bound of the stimulus from the stimulus location to the most distant stimulus with overlapping confidence intervals.  
 #' When `ref_lines = "all"`, all lines are plotted, though in displays with many stimuli, this can make for a messy graph.  When `"ref_lines = ambiguous"` is specified, then only the ones that help discriminate in cases where the result might be visually difficult to discern are plotted. 
@@ -495,13 +591,15 @@ make_segs <- function(.data, vdt = .02, ...){
 #' * `est` - The parameter estimate
 #' * `se` - The standard error of the estimate
 #' * `lwr`, `upr` - The inferential confidence bounds being used
+#' * `lwr_add`, `upr_add` - The confidence intervals that come from `add_level`. 
 #' * `label` - Factor giving the parameter names
 #' * `stim_start`, `stim_end` - y-axis bounds of the reference line
 #' * `bound_start`, `bound_end` - x-axis values for reference lines
 #' * `ambiguous` - Logical vector indicating whether the comparison is considered "ambiguous". 
 #' @method plot viztest
 #' @importFrom dplyr left_join arrange `%>%` join_by
-#' @importFrom ggplot2 ggplot geom_pointrange geom_segment aes
+#' @importFrom ggplot2 ggplot geom_pointrange geom_segment aes labs geom_point geom_linerange unit theme_bw scale_color_manual scale_linewidth_manual theme margin
+#' @importFrom ggtext element_textbox_simple
 #' @examples
 #' data(mtcars)
 #' mod2 <- lm(mpg ~ as.factor(cyl) + vs + am + as.factor(gear), data = mtcars)
@@ -509,7 +607,26 @@ make_segs <- function(.data, vdt = .02, ...){
 #' plot(v, ref_lines="ambiguous") + ggplot2::theme_classic()
 #' 
 #' @export
-plot.viztest <- function(x, ..., ref_lines="none", viz_diff_thresh = .02, make_plot=TRUE, level=c("ce","max","min","median"),trans=I){
+plot.viztest <- function(x, 
+                         ..., 
+                         add_test_level = TRUE, 
+                         ref_lines="none", 
+                         viz_diff_thresh = .02, 
+                         make_plot=TRUE, 
+                         level=c("ce","max","min","median"),
+                         trans=I, 
+                         est_point_args = list(color="black", size=2), 
+                         opt_ci_args = list(),
+                         test_ci_args = list(),
+                         ref_line_args = list(color="gray75", linetype=3),
+                         scale_linewidth_args = list(values=c(3.5, .5)),
+                         scale_color_args = list(values = c("gray75", "black")),
+                         overall_theme = theme_bw,
+                         theme_arg = list(legend.position="top", 
+                                          plot.caption = element_textbox_simple(width = unit(1, "npc"),  # Wraps to plot width
+                                                                                halign = 0,
+                                                                                margin = margin(1, 0, 0, 0,"lines"))),# Prevents overlap of caption and x-axis title
+                         remove_caption=FALSE){
   inp <- x$est
   tmp <- x$tab[which(x$tab$psame == max(x$tab$psame)), ]
   if(!is.numeric(level)){
@@ -531,25 +648,52 @@ plot.viztest <- function(x, ..., ref_lines="none", viz_diff_thresh = .02, make_p
   segs$vbl <- rownames(segs)
   inp$label <- factor(1:nrow(inp), labels=inp$vbl)
   inp <- left_join(inp, segs, by=join_by(vbl))
-  cols <- c("est","lwr","upr","bound_start","bound_end")
-  inp[,cols] <- apply(inp[,cols],2,trans)
+  inp[,c("est","lwr","upr","lwr_add","upr_add","bound_start","bound_end")] <- apply(inp[,c("est","lwr","upr","lwr_add","upr_add","bound_start","bound_end")],2,trans)
   if(any(inp$vbl == "zero"))inp <- inp[-which(inp$vbl == "zero"), ]
   if(!make_plot){
     res <- inp
   }else{
+    if(add_test_level){
+      opt_ci_args$mapping <- aes(x=est, xmin=lwr, xmax=upr, y=label, 
+                                 colour=sprintf('Optimal Visual Intervals (%.1f%%)', level*100), 
+                                 linewidth=sprintf('Optimal Visual Intervals (%.1f%%)', level*100))
+      test_ci_args$mapping <- aes(xmin = lwr_add, xmax=upr_add, y=label, 
+                                  colour=sprintf('Standard Confidence Intervals (%.1f%%)', (1-attr(x, "test_level"))*100), 
+                                  linewidth=sprintf('Standard Confidence Intervals (%.1f%%)', (1-attr(x, "test_level"))*100))
+      lab_args <- list(x="Estimate", y= "Parameter", colour="", linewidth="")
+    }else{
+      opt_ci_args$mapping <- aes(x=est, xmin=lwr, xmax=upr, y=label)
+      lab_args <- list(x=sprintf("Estimate and Optimal Visual CI (%.1f%%)", 
+                               level*100), 
+                       y = "Parameter")
+    }
+    est_point_args$mapping <- aes(x=est, y=label)
+    ref_line_args$mapping <- aes(x=bound_start, xend=bound_end, y=stim_start, yend=stim_end)
     g <- ggplot(inp) + 
-      geom_pointrange(aes(x=est, xmin = lwr, xmax=upr, y=label), size=.01) 
+      do.call(geom_linerange, opt_ci_args) +
+      do.call(scale_linewidth_manual,scale_linewidth_args) +
+      do.call(scale_color_manual,scale_color_args)+
+      overall_theme() +
+      do.call(theme,theme_arg)
+    if(add_test_level & all(c("lwr_add", "upr_add") %in% colnames(inp))){
+      g <- g + do.call(geom_linerange, test_ci_args)
+    }
+    g <- g + do.call(geom_point, est_point_args)
     if("all" %in% ref_lines){
-      g <- g + geom_segment(aes(x=bound_start, xend=bound_end, y=stim_start, yend=stim_end), color="gray75", linetype=3)
+      g <- g + do.call(geom_segment, ref_line_args)
     }  
     if( "ambiguous" %in% ref_lines){
-      g <- g + geom_segment(data = inp[which(inp$ambiguous), ], aes(x=bound_start, xend=bound_end, y=stim_start, yend=stim_end), color="gray75", linetype=3)
+      ref_line_args$data <- inp[which(inp$ambiguous), ]
+      g <- g + do.call(geom_segment, ref_line_args)
     }
     if(!any(c("ambiguous", "all", "none") %in% ref_lines)){
       incl <- which(ref_lines %in% inp$vbl)
       if(length(incl) == 0)stop("ref_lines should either be one of (all, ambiguous, or none) or a vector of names consistent with x$est$vbl.\n")
-      g <- g + geom_segment(data = inp[incl, ], aes(x=bound_start, xend=bound_end, y=stim_start, yend=stim_end), color="gray75", linetype=3)
+      ref_line_args$data <- inp[incl, ]
+      g <- g + do.call(geom_segment, ref_line_args)
     }
+    if(!remove_caption)lab_args$caption <- "Confidence intervals have been adjusted to permit visual testing as per Armstrong and Poirier (2025) [doi:10.1017/pan.2024.24]"
+    g <- g + do.call(labs, lab_args)
     res <- g
   }
   return(res)
@@ -667,3 +811,212 @@ gen_z <- function(b, v, alpha=.05, df = Inf, ...){
   return(list(ave_z = tmp, all_z = eg))
 }
 
+
+#' Internal function to check for symmetry
+#' @noRd
+is_sym <- function(x, tol=1e-08){
+  rnd <- abs(floor(log10(abs(tol))))
+  x <- round(x,rnd)
+  sqr <- nrow(x) == ncol(x) 
+  if(sqr){
+    lt_ut <- all(x[lower.tri(x)] == t(x)[lower.tri(x)])
+  }else{
+    lt_ut <- FALSE
+  }
+  lt_ut
+}
+
+#' Internal function to check for positive definiteness
+#' @noRd
+is_pd <- function(x, tol=1e-08){
+  if(is_sym(x, tol = tol) & is.numeric(x)){
+    ev <- eigen(x, only.values = TRUE)$values
+    ev <- ifelse(abs(ev) < tol, 0, ev)
+    !any(ev < 0)
+  }else{
+    FALSE
+  }
+}
+
+
+#' Get Letters for Multiple Comparisons
+#' 
+#' Gets the letter matrix for a compact letter display.  This can be passed to the `letter_plot()` function from the `psre` package to produce plots of 
+#' confidence intervals with a letter display.  
+#' 
+#' @param x An object that can be one of the following classes: an object of class `glht` produced by `glht()` from the `multcomp` package, 
+#' an object of class `emmGrid` produced by the `emmeans` package, or a list with elements `est` - a vector of estimates and `var` - a variance-covariance matrix for the estimates.
+#' @param ... Additional arguments passed down either to `cld` if the object is an `emmGrid` class or `summary.glht` if the object is a `glht` class.  
+#' If `x` is a list of estimates and variances, it will be converted to an `emmGrid` object internally and the `emmGrid` method dispatched, 
+#' `...` will be passed to `cld` in that case. Additional arguments for the `summary.glht()` include `test` and a host of others.  See the help file for `summary.glht` for examples. 
+#' Additional arguments for `cld` include `adjust` for multiplicity corrections and others.  See `?emmeans:::cld.emmGrid` for options and details. 
+#' 
+#' @returns A logical index indicating which estimates are in which letter group. 
+#' @export
+#' 
+#' @importFrom multcomp glht cld
+get_letters <- function(x=NULL, ...){
+  UseMethod("get_letters")
+}
+
+#' @importFrom emmeans emmobj
+#' @method get_letters default
+#' @export
+get_letters.default <- function(x, ...){
+  if(!all(c("est", "var") %in% names(x))){
+    stop("x should be a list with elements 'est' - a vector of estimates and 'var' - a variance-covariance matrix for the estimates.\n")
+  }
+  est <- x$est
+  nms <- names(est)
+  if(is.null(nms)){
+    nms <- paste0("b_", seq_along(est))
+    names(est) <- nms
+  }
+  obj <- emmobj(est, x$var, levels=nms)
+  get_letters(obj, ...)
+}
+
+#' @method get_letters glht 
+#' @export
+get_letters.glht <- function(x, ...){
+  args <- list(...)
+  args$object <- x
+  s <- do.call(summary, args)
+  cl <- cld(s)
+  cl$mcletters$LetterMatrix
+}
+
+#' @method get_letters emmGrid
+#' @export
+get_letters.emmGrid <- function(x, ...){
+  args <- list(...)
+  args$Letters <- letters
+  args$object <- x
+  cl <- do.call(cld, args)
+  grps <- cl$.group
+  unletters <- unique(c(unlist(strsplit(grps, ""))))
+  if(any(unletters == " ")){
+    unletters <- unletters[-which(unletters == " ")]
+  }
+  unletters <- sort(unletters)
+  out <- matrix(NA, nrow = nrow(cl), ncol=length(unletters))
+  colnames(out) <- unletters
+  rownames(out) <- cl[[1]]
+  for(l in seq_along(unletters)){
+    out[,l] <- grepl(unletters[l], grps)
+  }
+  out
+}
+
+
+
+#' Make Annotations for Significance Brackets
+#' 
+#' Makes a list of annotations for significance brakcets produced by the `geom_signif()` function from the `ggsignif` package.  The annotations are added for 
+#' pairs of estimates whose confidence intervals overlap, but the estimates are nonetheless significantly different from each other.  
+#' 
+#' @param obj An object of class `viztest` produced by the `viztest()` function.
+#' @param type Indicates whether annotations are produced for overlapping intervals that are significantly different from each other or not. The `"auto"` option will
+#' find the type that produces the fewest annotations. If `type="discrepancies"`, annotations will be made for pairs of estimates whose test results do not correspond with the (non-)overlaps in the confidence intervals. 
+#' @param tol Tolerance for determining whether intervals are close enough to be considered ambiguous.  This also plots significance flags for intervals that 
+#' do not overlap, but the distance between them is smaller than the tolerance.  The default is zero, but increasing the value will potentially produce more significance flags. 
+#' @param nudge A vector of the same length as the number of brackets.  This will nudge the y-position of the brakcet by the indicated amount.  This will be difficult to 
+#' specify ahead of time, but can be specified to clean up a plot after an initial run. 
+#' @param ... Other arguments, currently ignored. 
+#' 
+#' @importFrom stats confint
+#' @export
+#' @examples
+#' data(chickwts)
+#' chick_mod <- lm(weight ~ feed, data=chickwts)
+#' library(marginaleffects)
+#' chick_preds <- avg_predictions(chick_mod, variables="feed")
+#' b <- coef(chick_preds)
+#' names(b) <- chick_preds$feed
+#' v <- vcov(chick_preds)
+#' chick_vt_data <- make_vt_data(b, v)
+#' chick_vt <- viztest(chick_vt_data, test_level = 0.0001, include_zero=FALSE)
+#' chick_vt
+#' 
+#' make_annotations(chick_vt, type="discrepancies")
+make_annotations <- function(obj, type = c("auto", "significant", "insignificant", "discrepancies"),  tol = 0, nudge=NULL, ...){
+  typ <- match.arg(type)
+  if(typ == "auto"){
+    ns <- make_annotations(obj, type="insignificant", tol=tol, nudge=nudge, ...)
+    s <- make_annotations(obj, type="significant", tol=tol, nudge=nudge, ...)
+    if(length(s$annotations) > length(ns$annotations)){
+      return(ns)
+    }else{
+      return(s)
+    }    
+  }
+  lwr <- obj$est$lwr_add
+  names(lwr) <- obj$est$vbl
+  upr <- obj$est$upr_add
+  names(upr) <- obj$est$vbl
+  rng <- max(upr) - min(lwr)
+  if(typ != "discrepancies"){
+    tmp_combs <- data.frame(
+      smaller = obj$param_names[obj$combs[2,]], 
+      larger = obj$param_names[obj$combs[1,]] 
+    ) %>% 
+      mutate(us = upr[smaller], 
+             ll = lwr[larger],
+             ul = upr[larger], 
+             olap = ll < us) 
+    if(typ == "significant"){
+    tmp_combs <- tmp_combs %>%
+      mutate(s = obj$pw_test, 
+             ambig = ll > us & (ll - us) <= tol) %>% 
+      filter(s & (olap | ambig))
+    }else{
+      tmp_combs <- tmp_combs %>%
+        mutate(s = !obj$pw_test) %>% 
+        filter(s & olap)
+    }
+    if(is.null(nudge))nudge <- rep(0, nrow(tmp_combs))
+    if(length(nudge) != nrow(tmp_combs)){
+      warning("nudge must be NULL or same length as number of annotations, currently ignored")
+      nudge <- rep(0, nrow(tmp_combs))
+    } 
+    annot_flag <- ifelse(typ == "significant", "*", "NS")
+    list(
+      annotations = rep(annot_flag, nrow(tmp_combs)), 
+      y_position = tmp_combs$ul + .05*rng + nudge, 
+      xmin = tmp_combs$smaller, 
+      xmax = tmp_combs$larger
+    )
+  }else{
+    tab <- obj$tab
+    tab$row <- 1:nrow(tab)
+    use_levs <- tab[which(tab$psame == max(tab$psame)), ]
+    use_row <- use_levs[which.max(use_levs$easy), "row"]
+    tests <- obj$pw_test
+    olaps <- obj$ci_tests[,use_row]
+    discrep <- which(olaps != tests)
+    if(length(discrep) > 0){
+      dcombs <- obj$combs[, discrep, drop=FALSE]
+      tmp_combs <- data.frame(
+        smaller = obj$param_names[obj$combs[2,discrep]], 
+        larger = obj$param_names[obj$combs[1,discrep]] 
+      ) %>% 
+        mutate(ul = upr[larger]) 
+      if(is.null(nudge))nudge <- rep(0, nrow(tmp_combs))
+      if(length(nudge) != nrow(tmp_combs)){
+        warning("nudge must be NULL or same length as number of annotations, nudge being ignored")
+        nudge <- rep(0, nrow(tmp_combs))
+      } 
+      
+      list(
+        annotations = ifelse(tests[discrep], "*", "NS"), 
+        y_position = tmp_combs$ul + .05*rng + nudge, 
+        xmin = tmp_combs$smaller, 
+        xmax = tmp_combs$larger
+      )
+    }else{
+      stop("No discrepancies between pairwise tests and (non-)overlaps in inferential confidence intervals.\n")
+    }
+  }
+  
+}
+
